// **** START of PARAMETERS - EDIT HERE ********************************

// **** GEOMETRY ****
define(ox, 0.)// coordinates of left bottom front corner
define(oy, 0.)
define(oz, 0.)

define(w, 2.)// flow domain width (y axis)
define(l, 4.)// its length (x axis)
define(hf, 1.)// and height (z axis)

define(fx, 1.6)// (fx,fy) is a planar vector from (ox,oy) to cylinder axis 
define(fy, 1.)

define(df, 0.5)// cylinder diameter
define(dsz, 1.2)// diameter of rounded part of the mesh

// **** MESH ****
// computational cell size at the cylinder footing
define(ms, 0.03)
// mesh expansion ratio - mesh size grows with distance from cylinder
define(me, 2)

// you can play with expansion ratios for directions and different blocks 
// they default to 'me' value
define(nxur, me) // negative x - blocks 0,4,10,16
define(nysr, me) // negative y - blocks 0,1,2,3 and positive y - b.16,17,18,19
define(nrr, me) // from the cylinder - b. 5,6,7,8,11,12,13,14
define(nxdr, me) // positive x - blocks 3,9,15,19
define(ntzr, me) // positive z - all blocks

// **** END of PARAMETERS - DON'T CHANGE ANYTHING BELOW ****************

/*---------------------------------------------------------------------------*\
| =========                 |                                                 |
| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\    /   O peration     | Version:  1.0                                   |
|   \\  /    A nd           | Web:      http://www.openfoam.org               |
|    \\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/

FoamFile
{
    version         2.0;
    format          ascii;

    root            "";
    case            "";
    instance        "";
    local           "";

    class           dictionary;
    object          blockMeshDict;
}

// * * * *  check the OpenFOAM Wiki: HowTo_blockMesh_with_m4 * * * * * * * * //

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
 
changecom(//)changequote([,])
define(calc, [esyscmd(perl -e 'printf ($1)')])
define(calcint, [esyscmd(perl -e 'printf ("%.0f",$1)')])
define(VCOUNT, 0)
define(vlabel, [[// ]Vertex $1 = VCOUNT define($1, VCOUNT)define([VCOUNT], incr(VCOUNT))])
define(hexzt, hex ($1z $2z $3z $4z $1t $2t $3t $4t))
define(hexbz, hex ($1b $2b $3b $4b $1z $2z $3z $4z))
define(s22, 0.3827)
define(c22, 0.9239)
define(tzPatch, ($1z $2z $2t $1t))
define(zPatch, ($1z $2z $3z $4z))
define(tPatch, ($1t $2t $3t $4t))
convertToMeters 1;



define(cfx, calc(ox+fx))
define(cfy, calc(oy+fy))
define(rzd, calc(dsz/2*0.7071))
define(rzm, calc(df/2*0.7071))
define(zz, oz)
define(zt, calc(oz+hf))
//nxu
define(l1,calc(ms/(fx-0.5*dsz)*nxur))
define(xratio1, nxur)
define(nxu, calcint(log(xratio1*(l1-1)/(xratio1*l1-1))/log((l1-1)/(xratio1*l1-1))))
//nxd
define(l2,calc(ms*nrr/(l - fx - 0.5*dsz)))
define(xratio2, nxdr)
define(nxd, calcint(log(xratio2*(l2-1)/(xratio2*l2-1))/log((l2-1)/(xratio2*l2-1))))
//nys
define(l3,calc(ms*nrr/(fy - 0.5*dsz)))
define(xratio3, nysr)
define(nys, calcint(log(xratio3*(l3-1)/(xratio3*l3-1))/log((l3-1)/(xratio3*l3-1))))
//nr
define(l4,calc(ms/((dsz-df)/2)))
define(xratio4, nrr)
define(nr, calcint(log(xratio4*(l4-1)/(xratio4*l4-1))/log((l4-1)/(xratio4*l4-1))))
//na
define(na, calcint((3.14*df/8)/ms ))
//nzt
define(l5,calc(ms/hf))
define(xratio5, ntzr)
define(nzt, calcint(log(xratio5*(l5-1)/(xratio5*l5-1))/log((l5-1)/(xratio5*l5-1))))

vertices
(
// z level (bottom)
	(ox oy zz) vlabel(o0z)
	(calc(cfx - rzd) oy zz) vlabel(o1z)
	(cfx oy zz) vlabel(o2z)
	(calc(cfx+rzd) oy zz) vlabel(o3z)
	(calc(ox+l) oy zz) vlabel(o4z)

	(ox calc(cfy - rzd) zz) vlabel(o5z)
	(calc(cfx - rzd) calc(cfy - rzd) zz) vlabel(o6z)
	(cfx calc(cfy - dsz/2) zz) vlabel(o7z)
	(calc(cfx+rzd) calc(cfy - rzd) zz) vlabel(o8z)
	(calc(ox+l) calc(cfy - rzd) zz) vlabel(o9z)
	
	(ox cfy zz) vlabel(o10z)
	(calc(cfx - dsz/2) cfy zz) vlabel(o11z)
	(calc(cfx - df/2) cfy zz) vlabel(o12z)
	(calc(cfx - rzm) calc(cfy - rzm) zz) vlabel(o13z)
	(cfx calc(cfy - df/2) zz) vlabel(o14z)
	(calc(cfx+rzm) calc(cfy - rzm) zz) vlabel(o15z)
	(calc(cfx+df/2) cfy zz) vlabel(o16z)
	(calc(cfx+dsz/2) cfy zz) vlabel(o17z)
	(calc(ox+l) cfy zz) vlabel(o18z)

	(ox calc(cfy+rzd) zz) vlabel(o19z)
	(calc(cfx - rzd) calc(cfy+rzd) zz) vlabel(o20z)
	(calc(cfx - rzm) calc(cfy+rzm) zz) vlabel(o21z)
	(cfx calc(cfy+df/2) zz) vlabel(o22z)
	(calc(cfx+rzm) calc(cfy+rzm) zz) vlabel(o23z)
	(calc(cfx+rzd) calc(cfy+rzd) zz) vlabel(o24z)
	(calc(ox+l) calc(cfy+rzd) zz) vlabel(o25z)
	(cfx calc(cfy+dsz/2) zz) vlabel(o26z)
 
	(ox calc(oy+w) zz) vlabel(o27z)
	(calc(cfx - rzd) calc(oy+w) zz) vlabel(o28z)
	(cfx calc(oy+w) zz) vlabel(o29z)
	(calc(cfx+rzd) calc(oy+w) zz) vlabel(o30z)
	(calc(ox+l) calc(oy+w) zz) vlabel(o31z)

// t level (top)
	(ox oy zt) vlabel(o0t)
	(calc(cfx - rzd) oy zt) vlabel(o1t)
	(cfx oy zt) vlabel(o2t)
	(calc(cfx+rzd) oy zt) vlabel(o3t)
	(calc(ox+l) oy zt) vlabel(o4t)

	(ox calc(cfy - rzd) zt) vlabel(o5t)
	(calc(cfx - rzd) calc(cfy - rzd) zt) vlabel(o6t)
	(cfx calc(cfy - dsz/2) zt) vlabel(o7t)
	(calc(cfx+rzd) calc(cfy - rzd) zt) vlabel(o8t)
	(calc(ox+l) calc(cfy - rzd) zt) vlabel(o9t)
	
	(ox cfy zt) vlabel(o10t)
	(calc(cfx - dsz/2) cfy zt) vlabel(o11t)
	(calc(cfx - df/2) cfy zt) vlabel(o12t)
	(calc(cfx - rzm) calc(cfy - rzm) zt) vlabel(o13t)
	(cfx calc(cfy - df/2) zt) vlabel(o14t)
	(calc(cfx+rzm) calc(cfy - rzm) zt) vlabel(o15t)
	(calc(cfx+df/2) cfy zt) vlabel(o16t)
	(calc(cfx+dsz/2) cfy zt) vlabel(o17t)
	(calc(ox+l) cfy zt) vlabel(o18t)

	(ox calc(cfy+rzd) zt) vlabel(o19t)
	(calc(cfx - rzd) calc(cfy+rzd) zt) vlabel(o20t)
	(calc(cfx - rzm) calc(cfy+rzm) zt) vlabel(o21t)
	(cfx calc(cfy+df/2) zt) vlabel(o22t)
	(calc(cfx+rzm) calc(cfy+rzm) zt) vlabel(o23t)
	(calc(cfx+rzd) calc(cfy+rzd) zt) vlabel(o24t)
	(calc(ox+l) calc(cfy+rzd) zt) vlabel(o25t)
	(cfx calc(cfy+dsz/2) zt) vlabel(o26t)
 
	(ox calc(oy+w) zt) vlabel(o27t)
	(calc(cfx - rzd) calc(oy+w) zt) vlabel(o28t)
	(cfx calc(oy+w) zt) vlabel(o29t)
	(calc(cfx+rzd) calc(oy+w) zt) vlabel(o30t)
	(calc(ox+l) calc(oy+w) zt) vlabel(o31t)
);


blocks
(
	// 0
	hexzt(o0, o1, o6, o5) (nxu nys nzt) simpleGrading (calc(1/nxur) calc(1/nysr) ntzr)
	// 1
	hexzt(o1, o2, o7, o6) (na nys nzt) simpleGrading (1 calc(1/nysr) ntzr)
	// 2
	hexzt(o2, o3, o8, o7) (na nys nzt) simpleGrading (1 calc(1/nysr) ntzr)
	// 3
	hexzt(o3, o4, o9, o8) (nxd nys nzt) simpleGrading (nxdr calc(1/nysr) ntzr)
	// 4
	hexzt(o5, o6, o11, o10) (nxu na nzt) simpleGrading (calc(1/nxur) 1 ntzr)
	// 5
	hexzt(o12, o11, o6, o13) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 6
	hexzt(o13, o6, o7, o14) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 7
	hexzt(o14, o7, o8, o15) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 8
	hexzt(o15, o8, o17, o16) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 9
	hexzt(o8, o9, o18, o17) (nxd na nzt) simpleGrading (nxdr 1 ntzr)
	// 10
	hexzt(o10, o11, o20, o19) (nxu na nzt) simpleGrading (calc(1/nxur) 1 ntzr)
	// 11
	hexzt(o21, o20, o11, o12) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 12
	hexzt(o22, o26, o20, o21) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 13
	hexzt(o23, o24, o26, o22) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 14
	hexzt(o16, o17, o24, o23) (nr na nzt) simpleGrading (nrr 1 ntzr)
	// 15
	hexzt(o17, o18, o25, o24) (nxd na nzt) simpleGrading (nxdr 1 ntzr)
	// 16
	hexzt(o19, o20, o28, o27) (nxu nys nzt) simpleGrading (calc(1/nxur) nysr ntzr)
	// 17
	hexzt(o20, o26, o29, o28) (na nys nzt) simpleGrading (1 nysr ntzr)
	// 18
	hexzt(o26, o24, o30, o29) (na nys nzt) simpleGrading (1 nysr ntzr)
	// 19
	hexzt(o24, o25, o31, o30) (nxd nys nzt) simpleGrading (nxdr nysr ntzr)
);

edges		   
(
	arc o11z o6z (calc(cfx - c22*dsz/2) calc(cfy-s22*dsz/2) zz)
	arc o6z o7z (calc(cfx-s22*dsz/2) calc(cfy-c22*dsz/2) zz)
	arc o7z o8z (calc(cfx+s22*dsz/2) calc(cfy-c22*dsz/2) zz)
	arc o8z o17z (calc(cfx+c22*dsz/2) calc(cfy-s22*dsz/2) zz)
	arc o17z o24z (calc(cfx+c22*dsz/2) calc(cfy+s22*dsz/2) zz)
	arc o24z o26z (calc(cfx+s22*dsz/2) calc(cfy+c22*dsz/2) zz)
	arc o26z o20z (calc(cfx-s22*dsz/2) calc(cfy+c22*dsz/2) zz)
	arc o20z o11z (calc(cfx-c22*dsz/2) calc(cfy+s22*dsz/2) zz)

	arc o11t o6t (calc(cfx-c22*dsz/2) calc(cfy-s22*dsz/2) zt)
	arc o6t o7t (calc(cfx-s22*dsz/2) calc(cfy-c22*dsz/2) zt)
	arc o7t o8t (calc(cfx+s22*dsz/2) calc(cfy-c22*dsz/2) zt)
	arc o8t o17t (calc(cfx+c22*dsz/2) calc(cfy-s22*dsz/2) zt)
	arc o17t o24t (calc(cfx+c22*dsz/2) calc(cfy+s22*dsz/2) zt)
	arc o24t o26t (calc(cfx+s22*dsz/2) calc(cfy+c22*dsz/2) zt)
	arc o26t o20t (calc(cfx-s22*dsz/2) calc(cfy+c22*dsz/2) zt)
	arc o20t o11t (calc(cfx-c22*dsz/2) calc(cfy+s22*dsz/2) zt)

	arc o12z o13z (calc(cfx-c22*df/2) calc(cfy-s22*df/2) zz)
	arc o13z o14z (calc(cfx-s22*df/2) calc(cfy-c22*df/2) zz)
	arc o14z o15z (calc(cfx+s22*df/2) calc(cfy-c22*df/2) zz)
	arc o15z o16z (calc(cfx+c22*df/2) calc(cfy-s22*df/2) zz)
	arc o16z o23z (calc(cfx+c22*df/2) calc(cfy+s22*df/2) zz)
	arc o23z o22z (calc(cfx+s22*df/2) calc(cfy+c22*df/2) zz)
	arc o22z o21z (calc(cfx-s22*df/2) calc(cfy+c22*df/2) zz)
	arc o21z o12z (calc(cfx-c22*df/2) calc(cfy+s22*df/2) zz)

	arc o12t o13t (calc(cfx-c22*df/2) calc(cfy-s22*df/2) zt)
	arc o13t o14t (calc(cfx-s22*df/2) calc(cfy-c22*df/2) zt)
	arc o14t o15t (calc(cfx+s22*df/2) calc(cfy-c22*df/2) zt)
	arc o15t o16t (calc(cfx+c22*df/2) calc(cfy-s22*df/2) zt)
	arc o16t o23t (calc(cfx+c22*df/2) calc(cfy+s22*df/2) zt)
	arc o23t o22t (calc(cfx+s22*df/2) calc(cfy+c22*df/2) zt)
	arc o22t o21t (calc(cfx-s22*df/2) calc(cfy+c22*df/2) zt)
	arc o21t o12t (calc(cfx-c22*df/2) calc(cfy+s22*df/2) zt)

);

patches
 
(
	patch inlet
	(
	tzPatch(o5,o0)
	tzPatch(o10,o5)
	tzPatch(o19,o10)
	tzPatch(o27,o19)
	)
 
	patch outlet
	(
	tzPatch(o9,o4)
	tzPatch(o18,o9)
	tzPatch(o25,o18)
	tzPatch(o31,o25)
	)	
 
	symmetryPlane sides
	(
	tzPatch(o0,o1)
	tzPatch(o1,o2)
	tzPatch(o2,o3)
	tzPatch(o3,o4)
	tzPatch(o27,o28)
	tzPatch(o28,o29)
	tzPatch(o29,o30)
	tzPatch(o30,o31)
	)
 
	wall bottom
	(
	zPatch(o0,o1,o6,o5)
	zPatch(o1,o2,o7,o6)
	zPatch(o2,o3,o8,o7)
	zPatch(o3,o4,o9,o8)
	zPatch(o5,o6,o11,o10)
	zPatch(o8,o9,o18,o17)
	zPatch(o10,o11,o20,o19)
	zPatch(o17,o18,o25,o24)
	zPatch(o19,o20,o28,o27)
	zPatch(o20,o26,o29,o28)
	zPatch(o26,o24,o30,o29)
	zPatch(o24,o25,o31,o30)
	zPatch(o12,o11,o6,o13)
	zPatch(o13,o6,o7,o14)
	zPatch(o14,o7,o8,o15)
	zPatch(o15,o8,o17,o16)
	zPatch(o16,o17,o24,o23)
	zPatch(o23,o24,o26,o22)
	zPatch(o22,o26,o20,o21)
	zPatch(o21,o20,o11,o12)
	)
 
	wall cylinder
	(
	tzPatch(o12,o13)
	tzPatch(o13,o14)
	tzPatch(o14,o15)
	tzPatch(o15,o16)
	tzPatch(o16,o23)
	tzPatch(o23,o22)
	tzPatch(o22,o21)
	tzPatch(o21,o12)
	)

	symmetryPlane top
	(
	tPatch(o0,o1,o6,o5)
	tPatch(o1,o2,o7,o6)
	tPatch(o2,o3,o8,o7)
	tPatch(o3,o4,o9,o8)
	tPatch(o5,o6,o11,o10)
	tPatch(o8,o9,o18,o17)
	tPatch(o10,o11,o20,o19)
	tPatch(o17,o18,o25,o24)
	tPatch(o19,o20,o28,o27)
	tPatch(o20,o26,o29,o28)
	tPatch(o26,o24,o30,o29)
	tPatch(o24,o25,o31,o30)
	tPatch(o11,o6,o13,o12)
	tPatch(o13,o6,o7,o14)
	tPatch(o14,o7,o8,o15)
	tPatch(o15,o8,o17,o16)
	tPatch(o16,o17,o24,o23)
	tPatch(o23,o24,o26,o22)
	tPatch(o22,o26,o20,o21)
	tPatch(o21,o20,o11,o12)
	
	)
);

mergePatchPairs
(
);

